﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace MosaicGen
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MosaicGeneratorMainWindow : Window
    {
        MosaicWorker msW = new MosaicWorker(14, 10);
        private Rectangle [,] rectangles = new Rectangle[14,10];
        double rectSize;
        bool changed = false;
        
        public MosaicGeneratorMainWindow()
        {
            InitializeComponent();
            List<Msquare> squares = msW.SquareType;
            foreach (Msquare sq in squares)
            {
               Image i = new Image();
                i.Source = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(sq.Picture.GetHbitmap(), IntPtr.Zero, System.Windows.Int32Rect.Empty, System.Windows.Media.Imaging.BitmapSizeOptions.FromEmptyOptions());
                SamplesListBox.Items.Add(i);
            }
            SamplesListBox.Focus();
            SamplesListBox.SelectedIndex = 0;
            SquareLoad(false);
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            SquareLoad(false);
        }

        private void SquareLoad(bool start)
        {
            msW.ReloadSquares();
            SquaresCanvas.Children.Clear();
            rectangles = new Rectangle [msW.GetColumns(), msW.GetRows()];
            rectSize = Math.Min((SquaresCanvas.Width) / rectangles.GetLength(0), (SquaresCanvas.Height) / rectangles.GetLength(1));
            List<Msquare> squares = msW.Available;
            ListBoxUpdate(); 
            if (msW.Available.Count>0) {
            int a = 0;

            for (int j = 0; j < rectangles.GetLength(1); j++)
            {
                for (int i = 0; i < rectangles.GetLength(0); i++)
                {
                    Msquare square = (Msquare)squares [a].Clone();
                    a++;
                    a = a % msW.Available.Count;
                    msW [i, j] = square;
                    Rectangle r = new Rectangle();
                    r.Tag = new Tuple<int, int>(i, j);
                    r.Height = rectSize;
                    r.Width = rectSize;
                    if (!start)
                    {
                        RectangleFill(r, square.Picture);
                    }
                    else
                    {
                        r.Fill = Brushes.White;
                    }
                    r.Stroke = Brushes.Black;
                    r.StrokeThickness = 0;
                    r.MouseEnter += Rect_MouseEnter;
                    r.MouseLeave += Rect_MouseLeave;
                    r.PreviewMouseLeftButtonDown += Rect_PreviewMouseLeftButtonDown;

                    rectangles [i, j] = r;
                    SquaresCanvas.Children.Add(r);
                    Canvas.SetLeft(r, rectSize * i);
                    Canvas.SetTop(r, rectSize * j);
                }
                changed = false;
                SquaresCanvas.UpdateLayout();
            }

            }
            else
            {
                MessageBox.Show("You dont have any mosaic part to work with, use Manage Square in Edit", "Warning", MessageBoxButton.OK, MessageBoxImage.Stop);
            }
        }

        private void Rect_MouseEnter(object sender, MouseEventArgs e)
        {
            ((Rectangle)sender).StrokeThickness = 1;
            ((Rectangle)sender).Focus();
        }

        private void Rect_MouseLeave(object sender, MouseEventArgs e)
        {
            ((Rectangle)sender).StrokeThickness = 0;
            
        }

        private void Rect_PreviewMouseLeftButtonDown(object sender, MouseEventArgs e)
        {

            if (SamplesListBox.Items.Count > 0)
            {
                Msquare square;
                if (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl))
                {
                    square = msW [((Tuple<int, int>)((Rectangle)sender).Tag).Item1, ((Tuple<int, int>)((Rectangle)sender).Tag).Item2];
                    square.Rotate(1);
                    RectangleFill(((Rectangle)sender), square.Picture);
                }
                else
                {
                    int selected = SamplesListBox.SelectedIndex;
                    square = msW.Available [selected];
                    msW [((Tuple<int, int>)((Rectangle)sender).Tag).Item1, ((Tuple<int, int>)((Rectangle)sender).Tag).Item2] = square;
                    RectangleFill(rectangles [((Tuple<int, int>)((Rectangle)sender).Tag).Item1, ((Tuple<int, int>)((Rectangle)sender).Tag).Item2], square.Picture);
                }
                changed = true;
            }
            else
            {
                MessageBox.Show("You dont have any mosaic part to work with, use Manage Square in Edit", "Warning", MessageBoxButton.OK, MessageBoxImage.Stop);
            }
        }

        private void ListBoxUpdate()
        {
            List<Msquare> squares = msW.Available;
            SamplesListBox.Items.Clear();
            foreach (Msquare sq in squares)
            {

                Image i = new Image();
                i.Source = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(sq.Picture.GetHbitmap(), IntPtr.Zero, System.Windows.Int32Rect.Empty, System.Windows.Media.Imaging.BitmapSizeOptions.FromEmptyOptions());
                SamplesListBox.Items.Add(i);
            }
            SamplesListBox.SelectedIndex = 0;
        }

        /// <summary>
        /// pomocna aby se mi to nepletlo
        /// </summary>
        /// <param name="r"></param>
        /// <param name="b"></param>
        private void RectangleFill(Rectangle r, System.Drawing.Bitmap b)
        {
            r.Fill = new ImageBrush(System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(b.GetHbitmap(), IntPtr.Zero, System.Windows.Int32Rect.Empty, System.Windows.Media.Imaging.BitmapSizeOptions.FromEmptyOptions()));
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {

            Save();
           
        }

        private void Window_SizeChanged(object sender, SizeChangedEventArgs e)
        {
           CanvasParrent.Width = this.Width - 184 - 70;
           CanvasParrent.Height = this.Height - 100 - 70;
           SquaresCanvas.Width = this.Width - 184 - 70;
           SquaresCanvas.Height = this.Height - 100 - 70;
            SamplesListBox.Height = this.Height - 200;
            ListBoxCanvas.Height = this.Height - 200;
            border.Height =this.Height - 100 - 70;
            border.Width = this.Width - 184 - 70;
            this.UpdateLayout();
        }

        private void MyCommandClose( object sender, ExecutedRoutedEventArgs e ) 
      {
          e.Handled = true;
          this.Close();
      }

        private void MyCommandSave(object sender, ExecutedRoutedEventArgs e)
         {
          Save();
          e.Handled = true;
         }

        private void MyCommandNew(object sender, ExecutedRoutedEventArgs e)
        {
            if (!changed || MessageBox.Show("New mosaic without saving?", "Attention", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
            {
                SquareLoad(false);
            }
        }

        private void Save()
      {
          // celý problém je tu ten, že to co chceme rendrovat, musíme oblait rodičem ostejné velikosti na stejném umístění
           //ideálně gridem
          Rect b = VisualTreeHelper.GetDescendantBounds(SquaresCanvas);

          var rtb = new RenderTargetBitmap((int)b.Width, //width
              (int)b.Height, //height
              96, //dpi x
              96, //dpi y
              PixelFormats.Pbgra32 // pixelformat
              );
          rtb.Render(SquaresCanvas);
            
          BitmapEncoder pngEncoder = new PngBitmapEncoder();
            pngEncoder.Frames.Add(BitmapFrame.Create(rtb));

          SaveFileDialog sf = new SaveFileDialog();
          sf.InitialDirectory = System.AppDomain.CurrentDomain.BaseDirectory;
          sf.Filter = "Png images| *.png";
          sf.DefaultExt = ".png";
          sf.AddExtension = true;
         

          if (sf.ShowDialog() == true)
          {
              using (System.IO.MemoryStream ms = new System.IO.MemoryStream())
              {

                  pngEncoder.Save(ms);
                  System.IO.File.WriteAllBytes(sf.FileName, ms.ToArray());
                  changed = false;
              }
          }
      }

        private void MenuItemCreateSquare_Click(object sender, RoutedEventArgs e)
        {
            Window1 w = new Window1(msW);
            w.ShowDialog();
            SquareLoad(false);
        }

       

        private void MenuItemSize_Click(object sender, RoutedEventArgs e)
        {
            SizeChanged s = new SizeChanged(msW,msW.GetRows(), msW.GetColumns());
            //SizeChanged s = new SizeChanged(msW);
            s.ShowDialog();
            SquareLoad(false);
        }

        private void MyCommandHelp(object sender, ExecutedRoutedEventArgs e)
        {
            HelpWindow hW = new HelpWindow
                (" Welcome to my program and its help. /n Basic shortcuts for this window:/n F1 - Help, Ctrl+S - save image/mosaic , Ctrl+Q -quit , Ctrl+N  Default Mosaic./n Attention in every window work quick help by pres F1, and quick close by Ctrl+Q./n If you want work with samples/small images itself (adding new, edit created, manage them then choose edit from top menu./n If you want work with size of you mosaic or enable/disable Bounded mosaic choose settings. When bounded is check it menas that autogenerator dont work wiht Up border and Down border as same (it can have different edges/borders), but when is unchecked then count as one and you can compose bigger mosaic from this one./n In File you find save,new,quit function./n Basic how to:/nLeft click on picture in the midle, you changed specific image to selected image in Listbox, Left click + CTRL rotate specific image./n Save button-saved, default button 'clean' to default and autogener creates some mosaik./nMore information in documentation.");
            hW.Show();
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            Msquare [,] test = new Msquare [msW.GetColumns(), msW.GetRows()];
            MosaicGenerator mG = new MosaicGenerator(test, msW.Available, MenuItemBorder.IsChecked);
            test = mG.Generate(0);
            SquaresCanvas.Children.Clear();
            rectangles = new Rectangle [msW.GetColumns(), msW.GetRows()];
            rectSize = Math.Min((SquaresCanvas.Width) / rectangles.GetLength(0), (SquaresCanvas.Height) / rectangles.GetLength(1));

            for (int j = 0; j < rectangles.GetLength(1); j++)
            {
                for (int i = 0; i < rectangles.GetLength(0); i++)
                {
                    if (test [i, j] == null)
                    {
                        msW [i, j] = msW.SquareType [0];
                    }
                    else
                    {
                        msW [i, j] = test [i, j];
                        Rectangle r = new Rectangle();
                        r.Tag = new Tuple<int, int>(i, j);
                        r.Height = rectSize;
                        r.Width = rectSize;
                        RectangleFill(r, msW [i, j].Picture);
                        r.Stroke = Brushes.Black;
                        r.StrokeThickness = 0;
                        r.MouseEnter += Rect_MouseEnter;
                        r.MouseLeave += Rect_MouseLeave;
                        r.PreviewMouseLeftButtonDown += Rect_PreviewMouseLeftButtonDown;

                        rectangles [i, j] = r;
                        SquaresCanvas.Children.Add(r);
                        Canvas.SetLeft(r, rectSize * i);
                        Canvas.SetTop(r, rectSize * j);
                    }
                    // Msquare square = new Msquare(test [a].Picture, squares [a].Name, squares [a].Up, squares [a].Down, squares [a].Left, squares [a].Right);
                   // a++;
                   // a = a % 2;
                    
                    
                }
                changed = false;
                SquaresCanvas.UpdateLayout();
            }
        }

        private void MenuItemManageSquare_Click(object sender, RoutedEventArgs e)
        {
            AvailableDisband aD = new AvailableDisband(msW);
            aD.ShowDialog();
            ListBoxUpdate();
        }

        private void MenuItemAbout_Click(object sender, RoutedEventArgs e)
        {
            HelpWindow h = new HelpWindow
           ("It's me Mario", true);
            h.ShowDialog();
        }

       

       
    }
}
