﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;

namespace MosaicGen
{
    public class Msquare : ICloneable
    {
        private Bitmap picture;
        public Bitmap Picture
        {
            get { return picture; }
        }

        private int rotation;

        public int Rotation
        {
            get { return rotation; }
        } 
      
        private int up, down, left, right;

        public int Right
        {
            get { return right; }
        }
        public int Down
        {
            get { return down; }
        }
        public int Left
        {
            get { return left; }
        }
        public int Up
        {
            get { return up; }
           
        }

        private string name;

        public string Name
        {
            get { return name; }
        }


        public Msquare(Bitmap picture,string name, int up, int down, int left, int right)
        {
            this.picture = new Bitmap(picture);
            this.name = name;
            this.up = up;
            this.down = down;
            this.left = left;
            this.right = right;
        }


        /// <summary>
        /// otáčení obázku a to posměru hodinových ručiček, 
        /// pokud vložíte číslo 4 a větší nebo menší než 0 pomocí module se to ošetří
        /// </summary>
        /// <param name="rotation">1 je o 90°, 2 o 180° atd</param>
        public void Rotate(int rotation)
        {
            this.rotation = rotation % 4;
            switch (this.rotation)
            {
                case 1: picture.RotateFlip(RotateFlipType.Rotate90FlipNone); EdgeRotation(rotation); break;
                case 2: picture.RotateFlip(RotateFlipType.Rotate180FlipNone); EdgeRotation(rotation); break;
                case 3: picture.RotateFlip(RotateFlipType.Rotate270FlipNone); EdgeRotation(rotation); break;
                default: break;
            }
        }

        /// <summary>
        /// otáčení hran aby pasovali
        /// </summary>
        /// <param name="rotation"></param>
        private void EdgeRotation(int rotation)
        {
            int temp;
            if (rotation == 1)
            {
                temp = left;
                left = down;
                down = right;
                right = up;
                up = temp;
            }
            else if (rotation == 2)
            {
                temp = left;
                left = right;
                right = temp;
                temp = up;
                up = down;
                down = temp;

            }
            else if (rotation == 3)
            {
                temp = up;
                up = right;
                right = down;
                down = left;
                left = temp;
            }
        }

        /// <summary>
        /// ptá se jestli jeho soused napravo sedí/pasuje
        /// </summary>
        /// <param name="square">soused napravo</param>
        /// <returns></returns>
        public bool RightFit(Msquare square)
        {
            return (Right == square.Left) || (square.Left == 0) || (Right == 0);
        }

        /// <summary>
        /// ptá se jestli jeho soused nalevo sedí/pasuje
        /// </summary>
        /// <param name="square">soused nalevo</param>
        /// <returns></returns>
        public bool LefttFit(Msquare square)
        {
            return Left == square.Right || (Left == 0) || (square.Right == 0);
        }

        /// <summary>
        /// ptá se jestli jeho soused pod ním sedí/pasuje
        /// </summary>
        /// <param name="square">soused pod ním</param>
        /// <returns></returns>
        public bool DownFit(Msquare square)
        {
            return Down == square.Up || (Down == 0) || (square.Up == 0);
        }

        /// <summary>
        /// ptá se jestli jeho soused nad nim sedí/pasuje
        /// </summary>
        /// <param name="square">soused nad nim</param>
        /// <returns></returns>
        public bool UpFit(Msquare square)
        {
            return Up == square.Down || (square.Down == 0) || (Up == 0);
        }

        public override string ToString()
        {
            return Name +";" +Up.ToString()+";"+Down.ToString()+";"+Left.ToString()+";"+Right.ToString();
        }

        /// <summary>
        /// staticka metoda pro zpracování obrázku, vezme jakýkoliv Image a vráti bitmapu
        /// vyhazuje vyjímku pokud obrázek není čtvercový, tj by se pro mozaiku nehodil
        /// </summary>
        /// <param name="image"> libovolný obrázek</param>
        /// <returns> obrázek v bitmape 32x32</returns>
        public static Bitmap Convert(Image image)
        {
            Bitmap picture = new Bitmap(image);
            if (picture.Width == picture.Height)
            {
                picture = new Bitmap(picture, new Size(50, 50));
            }
            else
            {
                throw new BadImageFormatException("not square");
            }
            return picture;
        }

        public object Clone()
        {
           return new Msquare(Picture, Name, Up, Down, Left, Right);
        }

        public override bool Equals(object obj)
        {
            if (obj == null)
            {
                return false;
            }
            Msquare p = obj as Msquare;
            if ((System.Object)p == null)
            {
                return false;
            }
            return Name.Equals(p.Name);
        }

        public override int GetHashCode()
        {
            return Name.GetHashCode();
        }
    }
}
