﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace MosaicGen
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        private List<Msquare> createdSquares = new List<Msquare>();
        MosaicWorker msW;
        Msquare newMsquare;
        bool changed = false;
        bool validate = true;
        
        
        public Window1()
        {
            InitializeComponent();
        }

        public Window1(MosaicWorker msW)
        {
            InitializeComponent();
            this.msW = msW;
            createdSquares = msW.Available;
            LoadListBox();
            changed = false;
        }

        

        public void LoadListBox()
        {
           createdSquares = msW.Available;
            SamplesListBox.Items.Clear();
            foreach (Msquare sq in createdSquares)
            {

                Image i = new Image();
                i.Source = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(sq.Picture.GetHbitmap(), IntPtr.Zero, System.Windows.Int32Rect.Empty, System.Windows.Media.Imaging.BitmapSizeOptions.FromEmptyOptions());
                SamplesListBox.Items.Add(i);
            }
            SamplesListBox.Focus();
            SamplesListBox.SelectedIndex = 0;
        }

        private void SamplesListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (SamplesListBox.Items.Count > 0)
            {
                Msquare selSquare = createdSquares [SamplesListBox.SelectedIndex];
                PreviwPicture.Source = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(selSquare.Picture.GetHbitmap(), IntPtr.Zero, System.Windows.Int32Rect.Empty, System.Windows.Media.Imaging.BitmapSizeOptions.FromEmptyOptions());
                PreviewDown.Text = selSquare.Down.ToString();
                PreviewUp.Text = selSquare.Up.ToString();
                PreviewLeft.Text = selSquare.Left.ToString();
                PreviewRight.Text = selSquare.Right.ToString();
                PreviewName.Text = selSquare.Name;
            }
        }

        private void MyCommandClose(object sender, ExecutedRoutedEventArgs e)
        {
            e.Handled = true;
            this.Close();
        }

        private void MyCommandSave(object sender, ExecutedRoutedEventArgs e)
        {
            
            if (!changed)
            {
                MessageBox.Show("No change, nothing to save","Information",MessageBoxButton.OK,MessageBoxImage.Information);
            }
            else if (newMsquare == null)
            {
                MessageBox.Show("LoadImage first","Watchout",MessageBoxButton.OK,MessageBoxImage.Exclamation);
            }else if (!validate) {
                MessageBox.Show("Not all numbers are correct","warning",MessageBoxButton.OK,MessageBoxImage.Warning);
            }
            else
            {
                newMsquare = new Msquare(newMsquare.Picture, NewName.Text, Int32.Parse(NewUp.Text), Int32.Parse(NewDown.Text), Int32.Parse(NewLeft.Text), Int32.Parse(NewRight.Text));
                if (msW.Loader.CanSave(newMsquare))
                {
                  SaveFileDialog sfDialog = new SaveFileDialog();

                    string s = System.AppDomain.CurrentDomain.BaseDirectory + @"samples\";
                    sfDialog.FileName = newMsquare.Name;
                    sfDialog.InitialDirectory = s;

                    
                    sfDialog.DefaultExt = "*.bmp";
                    sfDialog.Filter = "Bmp images| *.bmp";


                    Rect b = VisualTreeHelper.GetDescendantBounds(NewSquearePicture);
                    var rtb = new RenderTargetBitmap((int)b.Width, //width
                        (int)b.Height, //height
                        96, //dpi x
                        96, //dpi y
                        PixelFormats.Pbgra32 // pixelformat
                        );
                    rtb.Render(NewSquearePicture);
                    BitmapEncoder bmpEncoder = new BmpBitmapEncoder();
                    bmpEncoder.Frames.Add(BitmapFrame.Create(rtb));

                    if (sfDialog.ShowDialog() == true)
                    {
                        sfDialog.FileName = s + newMsquare.Name + ".bmp";
                        using (System.IO.MemoryStream ms = new System.IO.MemoryStream())
                        {

                            bmpEncoder.Save(ms);
                            System.IO.File.WriteAllBytes(sfDialog.FileName, ms.ToArray());
                           
                            System.IO.FileStream fs1 = new System.IO.FileStream( System.IO.Path.GetDirectoryName(sfDialog.FileName) +@"\"+System.IO.Path.GetFileNameWithoutExtension(sfDialog.FileName)+ ".txt",  System.IO.FileMode.OpenOrCreate, System.IO.FileAccess.Write);
                            System.IO.StreamWriter writer = new System.IO.StreamWriter(fs1);
                            writer.Write(newMsquare.ToString());
                            writer.Close();
                          //  System.IO.File.Create();
                          //  System.IO.File.WriteAllText(System.AppDomain.CurrentDomain.BaseDirectory + @"samples\" + newMsquare.Name + ".txt", newMsquare.ToString());
                        }
                        msW.ReloadSquares();
                        LoadListBox();
                        changed = false;
                    }
                  //  msW.Loader.Save(newMsquare, NewSquearePicture);
                    msW.ReloadSquares();
                    LoadListBox();
                    changed = false;
                }
                else
                {
                    MessageBox.Show("jmeno už použito");
                }
            }
            e.Handled = true;
           
        }

        private void MyCommandNew(object sender, ExecutedRoutedEventArgs e)
        {
            if (!changed || MessageBox.Show("New mosaic without saving?", "Attention", MessageBoxButton.YesNo,MessageBoxImage.Question) == MessageBoxResult.Yes)
            {
                OpenFileDialog openfDial = new OpenFileDialog();
                openfDial.Filter = "Image Files(*.BMP;*.JPG;*.GIF;.*JPEG;*.TIFF;*.PNG)|*.TIFF;*.JPEG;*.BMP;*.JPG;*.GIF;*.PNG" ;
                if (openfDial.ShowDialog() == true)
                {
                    
                    System.Drawing.Bitmap image = new System.Drawing.Bitmap(openfDial.FileName);
                    try
                    {
                        image = Msquare.Convert(image);
                        newMsquare = new Msquare(image, "NewName1", 0, 0, 0, 0);
                        NewSquearePicture.Source= System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(image.GetHbitmap(), IntPtr.Zero, System.Windows.Int32Rect.Empty, System.Windows.Media.Imaging.BitmapSizeOptions.FromEmptyOptions());
                        changed = true;
                    }
                    catch (BadImageFormatException)
                    {
                        MessageBox.Show("Needs to be squared image", "warning",MessageBoxButton.OK,MessageBoxImage.Warning);
                    }
                    
                    
                }

            }
        }

        private void TextChanged(object sender, TextChangedEventArgs e)
        {
            changed = true;
            ((TextBox)sender).Background = Brushes.White;
            validate = true;
        }

        private void NumberValidation(object sender, KeyboardFocusChangedEventArgs e)
        {
           int result;
           TextBox send = ((TextBox)sender);
           if (!Int32.TryParse(send.Text, out result))
           {
               validate = false;
               send.Background = Brushes.Red;
               send.Focus();
              
           }
        }

        private void CommandBinding_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            HelpWindow h = new HelpWindow
            ("This windows is for creating new samples/images./nJust find and image !must be squeared! and best is 64x64, upload by Load Image button and than choose name and its sides./n Sides/edges work, that same numbers fit together (you can control with list of created samples)./n It is better to have it sied symetric, becaufe if it is not, the mosaic won't fit./n If you load you own you can 'destroy' the automatic mosaic generator by giving bad number to not fit side/edge. At the and press save button which saved permanently you new sample to folder with others.n/ 0 is univerzal number anf fit with everyone so be careful./n Shorcuts:/n F1 - help, Ctrl+S -save, Ctrl+Q quit/close, Ctrl+N new sample");
            h.ShowDialog();
        }

        

    
    }
}
